@@ -3,9 +3,7 @@ from __future__ import print_function
 
 """
 TODOs:
-    - Atomic replacement of output
     - Do not destroy output file if R fails or produces no output
-    - Timeout of R call
     - Arbitrary arguments support
 """
 
@@ -33,9 +31,10 @@ def argparser():
                    help='Image background [default: white]')
     parser.add_argument('--timeout', type=int, default=100,
                 help='How long (in seconds) we give to R to compute the image [default: 100]')
+    parser.add_argument('--quiet', action='store_true',
+                help='Swallow log output')
     return parser
 
-
 def lock_output(f):
     try:
         fcntl.lockf(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
@@ -48,9 +47,29 @@ def lock_output(f):
 def wait_until_unlocked(f):
     fcntl.lockf(f, fcntl.LOCK_SH)
 
-def main():
+def atomically_replace_destination(source_path, destination_file):
+    if 'nt' == os.name:
+        # os.rename is not atomic on windows, and raises if target file is already available
+        destination_file.seek(0)
+        destination_file.truncate()
+        shutil.copy(source_path, destination_file.name)
+        os.unlink(source_path)
+        return
+    
+    os.rename(source_path, destination_file.name) # atomic
+
+def is_empty_or_not_existant(a_path):
+    if not path.exists(a_path):
+        return True
+    
+    info = os.stat(a_path)
+    return info.st_size == 0
+
+def main(override_arguments=None):
     logging.getLogger().setLevel(logging.INFO)
-    args = argparser().parse_args()
+    args = argparser().parse_args(args=override_arguments)
+    if args.quiet:
+        logging.getLogger().setLevel(logging.ERROR)
     
     if not lock_output(args.output_plot):
         logging.info("Output file is already locked, subscribing to result.")
@@ -81,10 +100,11 @@ def main():
         logging.warn("R failed: %s\n%s" % (exception.stdout, exception.stderr))
         return
     
-    args.output_plot.seek(0)
-    args.output_plot.truncate()
-    shutil.copy(temp_output, args.output_plot.name)
-    os.unlink(temp_output)
+    if is_empty_or_not_existant(temp_output):
+        return # don't destroy original file
+    
+    atomically_replace_destination(temp_output, args.output_plot)
+    
     os.rmdir(temp_dir)
 
 if __name__ == '__main__':
